/*LOGIN MODE MODAL RESIZING*/
.main-content.authentication {
    padding-top: 50px !important;
}

@media (max-height: 560px) {
    .main-content.authentication {
        padding-top: 11px !important;
        min-height: 540px;
    }    
}

.content {
    height: 100%;
}

/*ICONS SPACING*/
.mstr-icon .selector { /*Attributes and Metrics icons*/
    padding-left: 17px;
}
.filter-title-container .mstr-icon { /*filters icons*/
    padding-left: 1px;
    padding-right: 2px;
}

/* .project-chooser-header div button { BUTTON FOR SORTING APPLICATION
    background: none;
} */

/*BROWSING DATASETS ROWS*/
/*.ant-radio {} Radio button*/
/*.mstr-icon .dataset {} Dataset rows icon*/
/*.icon-column {}*/

.ant-table-body  { /*All table body*/
    padding-left: 2px;
}

/* .content .checkbox-list .mstr-icon.selector svg {
    padding-left: 4px;
}
.content .checkbox-list .mstr-icon.selector svg {
    margin-left: 5px;
} */

/* .browser-content .ant-table-header th, .browser-content .ant-table-tbody>tr>td {
    padding: 16px 6px;
} */