name             'connector_jupyter'
maintainer       'Someone'
maintainer_email 'someone@microstrategy.com'
description      'Installs/Configures connector_jupyter servers'
long_description 'Installs and configures the servers this project produces'
version          '1.0.0'