#!/usr/bin/env bash

export APPLICATION_SHORT_VERSION_PREFIX="10.6."
export APPLICATION_LONG_VERSION_PREFIX="${APPLICATION_SHORT_VERSION_PREFIX}0."

export GROUP_ID_BASE='com.microstrategy'
export ARTIFACT_ID_BASE='connector-jupyter'

export VAGRANT_BOXES_CENTOS_NAME='centos-7-2-x86_64'
export VAGRANT_BOXES_CENTOS_VERSION='1.0.2.next'

export VAGRANT_BOXES_WINDOWS_NAME='windows-2008-r2'
export VAGRANT_BOXES_WINDOWS_VERSION='1.0.4.next'

export VAGRANT_BOXES_OSX_NAME='osx-10.11.3'
export VAGRANT_BOXES_OSX_VERSION='1.0.0.next'
