/*jshint esversion: 6*/
.modal-footer{
    padding: 0px;
    display: none;
}
.modal.fade .modal-dialog {
    max-height: 661px;  
    max-width: 538px;
}
.modal-dialog {
    margin: 30px auto;
    height: 90%;
}
.modal-header{
    padding: 0px;
    border-bottom: none;
    position: absolute;
    top: -1px;
    right: 2px;
    float: right;
    z-index: 1;
}
.modal-header > button {
    padding: 10px 10px 0px 0px;
}
.modal-body{
    padding: 0px 0px 0px 0px;
    height: 100%;
}
.top-nav{
    padding-bottom: 10px;
}
.header{
    height: 50px;
}
.modal-content {
    height: 100%; 
}
.modal-body > div {
    height: inherit;
}
* {
    box-sizing: border-box;
}

@media (max-height: 560px) {
    .modal.fade .modal-dialog {
        top: -15px;
    }
}


/* IFRAME STYLE */
iframe {
    display: block;
    width: 100%;
    height: 100%;
    border: none;
    padding-top: 5px;
}